package hu.bme.mit.books.manager;

import hu.bme.mit.books.model.Book;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.databinding.observable.list.WritableList;

public class BookManager implements IBookManager{

	List<Book> books;
	WritableList writeableList;

	/**
	 * Instantiates a new book manager with three sample books.
	 */
	public BookManager() {
		books = new ArrayList<Book>();
		addBook("Terry Pratchett", "The Light Fantastic", 232, true);
		addBook("Neil Gaiman", "American Gods", 560, true);
		addBook("Test author", "Test Book", 123, false);
	}

	/**
	 * Gets the all books.
	 * 
	 * @return the all books
	 */
	@Override
	public List<Book> getAllBooks() {
		return new ArrayList<Book>(books);
	}

	public void addObservableBookList(WritableList list) {
		writeableList = list;
	}
	/**
	 * Gets all read books.
	 * 
	 * @return the read books
	 */
	@Override
	public List<Book> getReadBooks() {
		ArrayList<Book> readBooks = new ArrayList<Book>();
		for (Book book : books) {
			if (book.isRead())
				readBooks.add(book);
		}
		return readBooks;
	}

	/**
	 * Gets the book of an author.
	 * 
	 * @param author
	 *            the author
	 * @return the book of author
	 */
	@Override
	public List<Book> getBookOfAuthor(String author) {
		ArrayList<Book> readBooks = new ArrayList<Book>();
		for (Book book : books) {
			if (book.getAuthor().contentEquals(author))
				readBooks.add(book);
		}
		return readBooks;
	}

	/**
	 * Adds a book with a given author, title, page number to the manager, and
	 * sets whether it is read or not.
	 * 
	 * @param author
	 *            the author
	 * @param title
	 *            the title
	 * @param pages
	 *            the pages
	 * @param read
	 *            the read
	 */
	@Override
	public void addBook(String author, String title, int pages, boolean read) {
		final Book book = new Book();
		book.setAuthor(author);
		book.setTitle(title);
		book.setPagenum(pages);
		book.setRead(read);
		if (writeableList != null) {
			writeableList.getRealm().asyncExec(new Runnable() {
				
				@Override
				public void run() {
					writeableList.add(book);
				}
			});
		} 
		books.add(book);
	}
	
	@Override
	public void removeBook(final Book book) {
		if (writeableList != null) {
			writeableList.getRealm().asyncExec(new Runnable() {
				
				@Override
				public void run() {
					writeableList.remove(book);
				}
			});
		} 
		books.remove(book);
	}
}
